@@ -6,15 +6,11 @@ import inspect
 import logging
 import re
 import sys
-from argparse import Namespace
 from collections import defaultdict
 from collections.abc import Iterable, Iterator, MutableMapping, MutableSequence
-from functools import lru_cache
 from importlib import import_module
 from pathlib import Path
-from typing import Any, cast
-
-from ruamel.yaml.comments import CommentedMap, CommentedSeq
+from typing import TYPE_CHECKING, Any, cast
 
 import ansiblelint.skip_utils
 import ansiblelint.utils
@@ -26,12 +22,18 @@ from ansiblelint._internal.rules import (
     RuntimeErrorRule,
     WarningRule,
 )
-from ansiblelint.config import PROFILES, get_rule_config
+from ansiblelint.app import App, get_app
+from ansiblelint.config import PROFILES, Options, get_rule_config
 from ansiblelint.config import options as default_options
 from ansiblelint.constants import LINE_NUMBER_KEY, RULE_DOC_URL, SKIPPED_RULES_KEY
 from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable, expand_paths_vars
 
+if TYPE_CHECKING:
+    from ruamel.yaml.comments import CommentedMap, CommentedSeq
+
+    from ansiblelint.errors import RuleMatchTransformMeta
+
 _logger = logging.getLogger(__name__)
 
 match_types = {
@@ -58,7 +60,6 @@ class AnsibleLintRule(BaseRule):
         """Retrieve rule specific configuration."""
         return get_rule_config(self.id)
 
-    @lru_cache(maxsize=256)
     def get_config(self, key: str) -> Any:
         """Return a configured value for given key string."""
         return self.rule_config.get(key, None)
@@ -74,22 +75,23 @@ class AnsibleLintRule(BaseRule):
     # pylint: disable=too-many-arguments
     def create_matcherror(
         self,
-        message: str | None = None,
-        linenumber: int = 1,
+        message: str = "",
+        lineno: int = 1,
         details: str = "",
         filename: Lintable | None = None,
         tag: str = "",
+        transform_meta: RuleMatchTransformMeta | None = None,
     ) -> MatchError:
         """Instantiate a new MatchError."""
         match = MatchError(
             message=message,
-            linenumber=linenumber,
+            lineno=lineno,
             details=details,
-            filename=filename,
+            lintable=filename or Lintable(""),
             rule=copy.copy(self),
+            tag=tag,
+            transform_meta=transform_meta,
         )
-        if tag:
-            match.tag = tag
         # search through callers to find one of the match* methods
         frame = inspect.currentframe()
         match_type: str | None = None
@@ -105,13 +107,14 @@ class AnsibleLintRule(BaseRule):
 
     @staticmethod
     def _enrich_matcherror_with_task_details(
-        match: MatchError, task: dict[str, Any]
+        match: MatchError,
+        task: ansiblelint.utils.Task,
     ) -> None:
         match.task = task
         if not match.details:
             match.details = "Task/Handler: " + ansiblelint.utils.task_to_str(task)
-        if match.linenumber < task[LINE_NUMBER_KEY]:
-            match.linenumber = task[LINE_NUMBER_KEY]
+        if match.lineno < task[LINE_NUMBER_KEY]:
+            match.lineno = task[LINE_NUMBER_KEY]
 
     def matchlines(self, file: Lintable) -> list[MatchError]:
         matches: list[MatchError] = []
@@ -121,27 +124,29 @@ class AnsibleLintRule(BaseRule):
             if line.lstrip().startswith("#"):
                 continue
 
-            rule_id_list = ansiblelint.skip_utils.get_rule_skips_from_line(line)
+            rule_id_list = ansiblelint.skip_utils.get_rule_skips_from_line(
+                line,
+                lintable=file,
+            )
             if self.id in rule_id_list:
                 continue
 
             result = self.match(line)
             if not result:
                 continue
-            message = None
+            message = ""
             if isinstance(result, str):
                 message = result
             matcherror = self.create_matcherror(
                 message=message,
-                linenumber=prev_line_no + 1,
+                lineno=prev_line_no + 1,
                 details=line,
                 filename=file,
             )
             matches.append(matcherror)
         return matches
 
-    # pylint: disable=too-many-branches
-    def matchtasks(self, file: Lintable) -> list[MatchError]:  # noqa: C901
+    def matchtasks(self, file: Lintable) -> list[MatchError]:
         """Call matchtask for each task inside file and return aggregate results.
 
         Most rules will never need to override matchtasks because its main
@@ -155,48 +160,55 @@ class AnsibleLintRule(BaseRule):
         ):
             return matches
 
-        tasks_iterator = ansiblelint.yaml_utils.iter_tasks_in_file(file)
-        for raw_task, task, skipped_tags, error in tasks_iterator:
-            if error is not None:
+        for task in ansiblelint.utils.task_in_list(
+            data=file.data,
+            kind=file.kind,
+            file=file,
+        ):
+            if task.error is not None:
                 # normalize_task converts AnsibleParserError to MatchError
-                return [error]
+                return [task.error]
 
             if (
-                self.id in skipped_tags
-                or ("action" not in task)
-                or "skip_ansible_lint" in task.get("tags", [])
+                self.id in task.skip_tags
+                or ("action" not in task.normalized_task)
+                or "skip_ansible_lint" in task.normalized_task.get("tags", [])
             ):
                 continue
 
             if self.needs_raw_task:
-                task["__raw_task__"] = raw_task
+                task.normalized_task["__raw_task__"] = task.raw_task
 
             result = self.matchtask(task, file=file)
             if not result:
                 continue
 
             if isinstance(result, Iterable) and not isinstance(
-                result, str
+                result,
+                str,
             ):  # list[MatchError]
                 # https://github.com/PyCQA/pylint/issues/6044
                 # pylint: disable=not-an-iterable
                 for match in result:
-                    if match.tag in skipped_tags:
+                    if match.tag in task.skip_tags:
                         continue
-                    self._enrich_matcherror_with_task_details(match, task)
+                    self._enrich_matcherror_with_task_details(
+                        match,
+                        task,
+                    )
                     matches.append(match)
                 continue
             if isinstance(result, MatchError):
-                if result.tag in skipped_tags:
+                if result.tag in task.skip_tags:
                     continue
                 match = result
             else:  # bool or string
-                message = None
+                message = ""
                 if isinstance(result, str):
                     message = result
                 match = self.create_matcherror(
                     message=message,
-                    linenumber=task[LINE_NUMBER_KEY],
+                    lineno=task.normalized_task[LINE_NUMBER_KEY],
                     filename=file,
                 )
 
@@ -216,7 +228,7 @@ class AnsibleLintRule(BaseRule):
         if isinstance(yaml, str):
             if yaml.startswith("$ANSIBLE_VAULT"):
                 return []
-            return [MatchError(filename=file, rule=LoadingFailureRule())]
+            return [MatchError(lintable=file, rule=LoadingFailureRule())]
         if not yaml:
             return matches
 
@@ -242,7 +254,7 @@ class AnsibleLintRule(BaseRule):
 class TransformMixin:
     """A mixin for AnsibleLintRule to enable transforming files.
 
-    If ansible-lint is started with the ``--write`` option, then the ``Transformer``
+    If ansible-lint is started with the ``--fix`` option, then the ``Transformer``
     will call the ``transform()`` method for every MatchError identified if the rule
     that identified it subclasses this ``TransformMixin``. Only the rule that identified
     a MatchError can do transforms to fix that match.
@@ -309,7 +321,6 @@ class TransformMixin:
         for segment in yaml_path:
             # The cast() calls tell mypy what types we expect.
             # Essentially this does:
-            #   target = target[segment]
             if isinstance(segment, str):
                 target = cast(MutableMapping[str, Any], target)[segment]
             elif isinstance(segment, int):
@@ -318,14 +329,14 @@ class TransformMixin:
 
 
 # pylint: disable=too-many-nested-blocks
-def load_plugins(  # noqa: max-complexity: 11
+def load_plugins(
     dirs: list[str],
 ) -> Iterator[AnsibleLintRule]:
     """Yield a rule class."""
 
     def all_subclasses(cls: type) -> set[type]:
         return set(cls.__subclasses__()).union(
-            [s for c in cls.__subclasses__() for s in all_subclasses(c)]
+            [s for c in cls.__subclasses__() for s in all_subclasses(c)],
         )
 
     orig_sys_path = sys.path.copy()
@@ -347,12 +358,15 @@ def load_plugins(  # noqa: max-complexity: 11
         # or rules that do not have a valid id. For example, during testing
         # python may load other rule classes, some outside the tested rule
         # directories.
-        if getattr(rule, "id") and Path(inspect.getfile(rule)).parent.absolute() in [
-            Path(x).absolute() for x in dirs
-        ]:
-            if issubclass(rule, BaseRule) and rule.id not in rules:
-                rules[rule.id] = rule()
-    for rule in rules.values():  # type: ignore
+        if (
+            rule.id  # type: ignore[attr-defined]
+            and Path(inspect.getfile(rule)).parent.absolute()
+            in [Path(x).absolute() for x in dirs]
+            and issubclass(rule, BaseRule)
+            and rule.id not in rules
+        ):
+            rules[rule.id] = rule()
+    for rule in rules.values():  # type: ignore[assignment]
         if isinstance(rule, AnsibleLintRule) and bool(rule.id):
             yield rule
 
@@ -360,21 +374,30 @@ def load_plugins(  # noqa: max-complexity: 11
 class RulesCollection:
     """Container for a collection of rules."""
 
-    def __init__(
+    def __init__(  # pylint: disable=too-many-arguments
         self,
-        rulesdirs: list[str] | None = None,
-        options: Namespace = default_options,
+        rulesdirs: list[str] | list[Path] | None = None,
+        options: Options | None = None,
         profile_name: str | None = None,
+        *,
         conditional: bool = True,
+        app: App | None = None,
     ) -> None:
         """Initialize a RulesCollection instance."""
-        self.options = options
+        if options is None:
+            self.options = copy.deepcopy(default_options)
+            # When initialized without options argument we want it to always
+            # be offline as this is done only during testing.
+            self.options.offline = True
+        else:
+            self.options = options
         self.profile = []
+        self.app = app or get_app(offline=True)
+
         if profile_name:
             self.profile = PROFILES[profile_name]
-        if rulesdirs is None:
-            rulesdirs = []
-        self.rulesdirs = expand_paths_vars(rulesdirs)
+        rulesdirs_str = [] if rulesdirs is None else [str(r) for r in rulesdirs]
+        self.rulesdirs = expand_paths_vars(rulesdirs_str)
         self.rules: list[BaseRule] = []
         # internal rules included in order to expose them for docs as they are
         # not directly loaded by our rule loader.
@@ -384,9 +407,9 @@ class RulesCollection:
                 AnsibleParserErrorRule(),
                 LoadingFailureRule(),
                 WarningRule(),
-            ]
+            ],
         )
-        for rule in load_plugins(rulesdirs):
+        for rule in load_plugins(rulesdirs_str):
             self.register(rule, conditional=conditional)
         self.rules = sorted(self.rules)
 
@@ -395,10 +418,11 @@ class RulesCollection:
         if profile_name and not (self.options.list_rules or self.options.list_tags):
             filter_rules_with_profile(self.rules, profile_name)
 
-    def register(self, obj: AnsibleLintRule, conditional: bool = False) -> None:
+    def register(self, obj: AnsibleLintRule, *, conditional: bool = False) -> None:
         """Register a rule."""
         # We skip opt-in rules which were not manually enabled.
         # But we do include opt-in rules when listing all rules or tags
+        obj._collection = self  # pylint: disable=protected-access # noqa: SLF001
         if any(
             [
                 not conditional,
@@ -407,9 +431,8 @@ class RulesCollection:
                 obj.id in self.options.enable_list,
                 self.options.list_rules,
                 self.options.list_tags,
-            ]
+            ],
         ):
-            obj._collection = self  # pylint: disable=protected-access
             self.rules.append(obj)
 
     def __iter__(self) -> Iterator[BaseRule]:
@@ -428,7 +451,7 @@ class RulesCollection:
         """Combine rules."""
         self.rules.extend(more)
 
-    def run(  # noqa: max-complexity: 12
+    def run(
         self,
         file: Lintable,
         tags: set[str] | None = None,
@@ -449,10 +472,10 @@ class RulesCollection:
                 return [
                     MatchError(
                         message=str(exc),
-                        filename=file,
+                        lintable=file,
                         rule=LoadingFailureRule(),
                         tag=f"{LoadingFailureRule.id}[{exc.__class__.__name__.lower()}]",
-                    )
+                    ),
                 ]
 
         for rule in self.rules:
@@ -463,10 +486,13 @@ class RulesCollection:
                 or rule.has_dynamic_tags
                 or not set(rule.tags).union([rule.id]).isdisjoint(tags)
             ):
+                _logger.debug("Running rule %s", rule.id)
                 rule_definition = set(rule.tags)
                 rule_definition.add(rule.id)
                 if set(rule_definition).isdisjoint(skip_list):
                     matches.extend(rule.getmatches(file))
+            else:
+                _logger.debug("Skipping rule %s", rule.id)
 
         # some rules can produce matches with tags that are inside our
         # skip_list, so we need to cleanse the matches
@@ -477,9 +503,18 @@ class RulesCollection:
     def __repr__(self) -> str:
         """Return a RulesCollection instance representation."""
         return "\n".join(
-            [rule.verbose() for rule in sorted(self.rules, key=lambda x: x.id)]
+            [rule.verbose() for rule in sorted(self.rules, key=lambda x: x.id)],
         )
 
+    def known_tags(self) -> list[str]:
+        """Return a list of known tags, without returning no sub-tags."""
+        tags = set()
+        for rule in self.rules:
+            tags.add(rule.id)
+            for tag in rule.tags:
+                tags.add(tag)
+        return sorted(tags)
+
     def list_tags(self) -> str:
         """Return a string with all the tags in the RulesCollection."""
         tag_desc = {
@@ -493,6 +528,7 @@ class RulesCollection:
             "metadata": "Invalid metadata, likely related to galaxy, collections or roles",
             "opt-in": "Rules that are not used unless manually added to `enable_list`",
             "security": "Rules related o potentially security issues, like exposing credentials",
+            "syntax": "Related to wrong or deprecated syntax",
             "unpredictability": "Warn about code that might not work in a predictable way",
             "unskippable": "Indicate a fatal error that cannot be ignored or disabled",
             "yaml": "External linter which will also produce its own rule codes",
@@ -500,8 +536,12 @@ class RulesCollection:
 
         tags = defaultdict(list)
         for rule in self.rules:
+            # Fail early if a rule does not have any of our required tags
+            if not set(rule.tags).intersection(tag_desc.keys()):
+                msg = f"Rule {rule} does not have any of the required tags: {', '.join(tag_desc.keys())}"
+                raise RuntimeError(msg)
             for tag in rule.tags:
-                tags[tag].append(rule.id)
+                tags[tag] = list(rule.ids())
         result = "# List of tags and rules they cover\n"
         for tag in sorted(tags):
             desc = tag_desc.get(tag, None)
@@ -509,8 +549,7 @@ class RulesCollection:
                 result += f"{tag}:  # {desc}\n"
             else:
                 result += f"{tag}:\n"
-            # result += f"  rules:\n"
-            for name in tags[tag]:
+            for name in sorted(tags[tag]):
                 result += f"  - {name}\n"
         return result
 
@@ -533,18 +572,4 @@ def filter_rules_with_profile(rule_col: list[BaseRule], profile: str) -> None:
                 profile,
             )
             rule_col.remove(rule)
-        else:
-            for tag in ("opt-in", "experimental"):
-                if tag in rule.tags:
-                    _logger.debug(
-                        "Removing tag `%s` from `%s` rule because `%s` profile makes it mandatory.",
-                        tag,
-                        rule.id,
-                        profile,
-                    )
-                    rule.tags.remove(tag)
-                    # rule_col.rules.remove(rule)
-                    # break
-            if "opt-in" in rule.tags:
-                rule.tags.remove("opt-in")
     _logger.debug("%s/%s rules included in the profile", len(rule_col), total_rules)
