@@ -2,17 +2,19 @@
 from __future__ import annotations
 
 import logging
-from argparse import Namespace
-from typing import Union, cast
+from typing import TYPE_CHECKING, Union, cast
 
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 
-from ansiblelint.errors import MatchError
 from ansiblelint.file_utils import Lintable
 from ansiblelint.rules import AnsibleLintRule, TransformMixin
-from ansiblelint.runner import LintResult
 from ansiblelint.yaml_utils import FormattedYAML, get_path_to_play, get_path_to_task
 
+if TYPE_CHECKING:
+    from ansiblelint.config import Options
+    from ansiblelint.errors import MatchError
+    from ansiblelint.runner import LintResult
+
 __all__ = ["Transformer"]
 
 _logger = logging.getLogger(__name__)
@@ -31,21 +33,19 @@ class Transformer:
     pre-requisite for the planned rule-specific transforms.
     """
 
-    def __init__(self, result: LintResult, options: Namespace):
+    def __init__(self, result: LintResult, options: Options):
         """Initialize a Transformer instance."""
         self.write_set = self.effective_write_set(options.write_list)
 
         self.matches: list[MatchError] = result.matches
         self.files: set[Lintable] = result.files
 
-        file: Lintable
-        # pylint: disable=undefined-variable
         lintables: dict[str, Lintable] = {file.filename: file for file in result.files}
         self.matches_per_file: dict[Lintable, list[MatchError]] = {
             file: [] for file in result.files
         }
-
-        for match in self.matches:
+        not_ignored = [match for match in self.matches if not match.ignored]
+        for match in not_ignored:
             try:
                 lintable = lintables[match.filename]
             except KeyError:
@@ -100,6 +100,10 @@ class Transformer:
                     # This is an empty vars file or similar which loads as None.
                     # It is not safe to write this file or data-loss is likely.
                     # Only maps and sequences can preserve comments. Skip it.
+                    _logger.debug(
+                        "Ignored reformatting %s because current implementation in ruamel.yaml would drop comments. See https://sourceforge.net/p/ruamel-yaml/tickets/460/",
+                        file,
+                    )
                     continue
 
             if self.write_set != {"none"}:
@@ -133,11 +137,11 @@ class Transformer:
             if file_is_yaml and not match.yaml_path:
                 data = cast(Union[CommentedMap, CommentedSeq], data)
                 if match.match_type == "play":
-                    match.yaml_path = get_path_to_play(file, match.linenumber, data)
+                    match.yaml_path = get_path_to_play(file, match.lineno, data)
                 elif match.task or file.kind in (
                     "tasks",
                     "handlers",
                     "playbook",
                 ):
-                    match.yaml_path = get_path_to_task(file, match.linenumber, data)
+                    match.yaml_path = get_path_to_task(file, match.lineno, data)
             match.rule.transform(match, file, data)
